/*
 * For licensing terms, see the file COPYING
 * For contact information, see the file AUTHORS
 *
 */
#ifndef _STREAM_H_
#define _STREAM_H_
/* */
#include <stdlib.h>

/* allocate, deallocate and reallocate are the memory management functions
 * used by the stream functions. They are to be defined as macros.
 * That way, if you don't explicitly define them, the standard C malloc
 * library will be used.
 *
 * This allows you to provide your own allocation functions
 */


/*Define our own memory management functions here*/
#include "stuff.h"
#define allocate   fn_malloc
#define deallocate fn_free
#define reallocate fn_realloc

#ifndef allocate
#include <malloc.h>
#define MALLOC_H_DEF
#define allocate(x) malloc(x)
#endif

#ifndef deallocate

#ifndef MALLOC_H_DEF
#include <malloc.h>
#endif //MALLLOC_H_DEF

#define deallocate(x) free(x)
#endif //deallocate

#ifndef reallocate

#ifndef MALLOC_H_DEF
#include <malloc.h>
#endif //MALLOC_DEF_H

#define reallocate(x,y) realloc(x,y)
#endif //reallocate



typedef struct _STREAM
{
	size_t capacity; /* actual capacity of the buffer */
	size_t size; /* the largest offset written to in the buffer */
	int offset; /* current offset into the buffer */
	char * buffer; /* actual data buffer */
}STREAM;

/* stream manipulation functions */
STREAM * stream_create (size_t len);
void   stream_free     (STREAM *stream);
size_t stream_write    (STREAM *stream,char *data,size_t len);
int    stream_add_char (STREAM *stream,char ch);
char * stream_data     (STREAM *stream);
char * stream_copy_buffer (STREAM * stream);
#define stream_add_str(stream,data) stream_write(stream,data,strlen(data))

#endif //_STREAM_H_
